<?php

namespace Drupal\simple_oauth\Repositories;

use Drupal\Core\DependencyInjection\ClassResolverInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\simple_oauth\Entities\AccessTokenEntity;
use League\OAuth2\Server\Entities\AccessTokenEntityInterface;
use League\OAuth2\Server\Entities\ClientEntityInterface;
use League\OAuth2\Server\Repositories\AccessTokenRepositoryInterface;
use Symfony\Component\Serializer\SerializerInterface;

/**
 * The access token repository.
 */
class AccessTokenRepository implements AccessTokenRepositoryInterface {

  use RevocableTokenRepositoryTrait;

  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected SerializerInterface $serializer,
    protected ClassResolverInterface $classResolver,
  ) {
  }

  /**
   * The bundle ID.
   *
   * @var string
   */
  protected static string $bundleId = 'access_token';

  /**
   * The OAuth2 entity class name.
   *
   * @var string
   */
  protected static string $entityClass = AccessTokenEntity::class;

  /**
   * The OAuth2 entity interface name.
   *
   * @var string
   */
  protected static string $entityInterface = AccessTokenEntityInterface::class;

  /**
   * {@inheritdoc}
   */
  public function persistNewAccessToken(AccessTokenEntityInterface $access_token_entity) {
    $this->persistNew($access_token_entity);
  }

  /**
   * {@inheritdoc}
   */
  public function revokeAccessToken($token_id) {
    $this->revoke($token_id);
  }

  /**
   * {@inheritdoc}
   */
  public function isAccessTokenRevoked($token_id) {
    return $this->isRevoked($token_id);
  }

  /**
   * {@inheritdoc}
   */
  public function getNewToken(ClientEntityInterface $client_entity, array $scopes, $user_identifier = NULL) {
    $access_token = $this->classResolver->getInstanceFromDefinition($this::$entityClass);
    $access_token->setClient($client_entity);
    foreach ($scopes as $scope) {
      $access_token->addScope($scope);
    }
    $access_token->setUserIdentifier($user_identifier);

    return $access_token;
  }

}
